function finished = Pooling(descriptorPath, codebookPath, codebookFileName, ...
    codebookFileExpansion, featurePath, preservedDimensions, kGMM, pyramid)

GMM = cell(1, length(pyramid));
for p = 1: 1: length(pyramid)
    codebookName = [codebookFileName, num2str(kGMM), codebookFileExpansion];
    codebookFile = fullfile(codebookPath, codebookName);
    GMM{p} = load(codebookFile);
    GMM{p} = GMM{p}.GMM;
end

fprintf('The pooling step begins.\n');
categoryDirectories = dir(descriptorPath);
totalImages = 0;
categories = 0;
for i = 1: 1: length(categoryDirectories)
    categoryName = categoryDirectories(i).name;
    if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
        categories = categories + 1;
        fprintf('Category # %03d: %s, begins.\n', categories, categoryName);
        featureDirectory = fullfile(featurePath, categoryName);
        if (~isdir(featureDirectory))
            mkdir(featureDirectory);
        end
        imageDirectory = fullfile(descriptorPath, categoryName);
        imageFiles = dir(fullfile(imageDirectory, '*.mat'));
        images = length(imageFiles);
        for j = 1: 1: images
            path = fullfile(descriptorPath, categoryName, imageFiles(j).name);
            [~, name_] = fileparts(imageFiles(j).name);
            featureFile = fullfile(featureDirectory, [name_, '.mat']);
            if (~exist(featureFile, 'file'))
                SIFTDescriptor = load(path);
                SIFT = SIFTDescriptor.SIFT;
                SIFT.d = SIFT.d(:, 1: preservedDimensions);
                dimensions = kGMM * size(SIFT.d, 2) * 2;
                features = [];
                for p = 1: 1: size(pyramid, 1)
                    width = SIFT.width / pyramid(p, 1);
                    height = SIFT.height / pyramid(p, 2);
                    x = ceil(SIFT.l(:, 1) / width);
                    y = ceil(SIFT.l(:, 2) / height);
                    bin = (y - 1) * pyramid(p, 1) + x;
                    subDimensions = dimensions;
                    coef = zeros(1, subDimensions * pyramid(p, 1) * pyramid(p, 2));
                    for s = 1: 1: pyramid(p, 1) * pyramid(p, 2)
                        ind = find(bin == s);
                        coef_ = vl_fisher(double(SIFT.d(ind, :))', ...
                            GMM{p}.means, GMM{p}.covariances, GMM{p}.priors, 'Improved'); %#ok<FNDSB>
                        coef((s - 1) * subDimensions + 1: s * subDimensions) = coef_;
                    end
                    features = cat(2, features, coef);
                end
                features = features / ((sum(features .^ 2)) ^ (1 / 2)); %#ok<NASGU>
                save(featureFile, 'features');
                fprintf('    Image #%03d out of %d images of category %03d is finished.\n', j, images, categories);
            else
                %fprintf('    Image #%03d out of %d images of category %03d existed.\n', j, images, categories);
            end
        end
        totalImages = totalImages + images;
        fprintf('Category # %03d: %s, is finished, %04d images have been processed.\n', categories, categoryName, totalImages);
    end
end
fprintf('The pooling step is finished.\n');
finished = true;

end
