function [accuracy, finished] = Classifier(featurePath, splitPath, ...
    resultPath, resultFileName, resultFileExpansion, times, trainingSamplesPerCategory, C)

[feature, ~] = LoadFeatures(featurePath, false);
categories = feature.categories;
totalSamples = feature.images;
trainingSamples = trainingSamplesPerCategory * categories;
testingSamples = totalSamples - trainingSamples;

if (~isdir(splitPath))
    mkdir(splitPath);
end
splitFile = fullfile(splitPath, [num2str(times), 'x', num2str(trainingSamplesPerCategory), 'T.mat']);
if (~exist(splitFile, 'file'))
    split.testingSamplesPerCategory = zeros(1, categories);
    split.maxTestingSamplesPerCategory = 0;
    for i = 1: 1: categories
        split.testingSamplesPerCategory(i) = feature.cate(i).images - trainingSamplesPerCategory;
        if (split.maxTestingSamplesPerCategory < split.testingSamplesPerCategory(i))
            split.maxTestingSamplesPerCategory = split.testingSamplesPerCategory(i);
        end
    end
    split.trainingSample = zeros(times, categories, trainingSamplesPerCategory);
    split.testingSample = zeros(times, categories, split.maxTestingSamplesPerCategory);
    for t = 1: 1: times
        for i = 1: 1: categories
            perm = randperm(feature.cate(i).images);
            split.trainingSample(t, i, :) = perm(1: trainingSamplesPerCategory);
            split.testingSample(t, i, 1: feature.cate(i).images - trainingSamplesPerCategory) = ...
                perm(trainingSamplesPerCategory + 1: feature.cate(i).images);
        end
    end
    save(splitFile, 'split');
else
    split = load(splitFile);
    split = split.split;
end

finished_ = true;
for s = 1: 1: length(C)
    resultFile = fullfile(resultPath, [resultFileName num2str(C(s)) resultFileExpansion]);
    if (~exist(resultFile, 'file'))
        finished_ = false;
        break;
    end
end
if (~finished_)
    if (~isdir(resultPath))
        mkdir(resultPath);
    end
    testingSamplesPerCategory = split.testingSamplesPerCategory;
    maxTestingSamplesPerCategory = split.maxTestingSamplesPerCategory;
    len = length(C);
    accuracyPerCategory = zeros(len, times, categories);
    cateAcc = zeros(len, times);
    imageAcc = zeros(len, times);
    accuracy = zeros(1, len);
    for t = 1: 1: times
        fprintf('Testing process No.%d begins.\n', t);
        trainingSample = squeeze(split.trainingSample(t, :, :));
        testingSample = squeeze(split.testingSample(t, :, :));
        fprintf('Testing environment is successfully constructed.\n');
        fprintf('%d training samples (%d per category) and %d testing samples in total.\n', ...
            trainingSamples, trainingSamplesPerCategory, testingSamples);
        dimensions = feature.dimensions;
        trainingFeature = zeros(trainingSamples, dimensions);
        trainingLabel = zeros(1, trainingSamples);
        trainingCount = 0;
        for i = 1: 1: categories
            for j = 1: 1: trainingSamplesPerCategory
                trainingCount = trainingCount + 1;
                vector = load(feature.cate(i).image(trainingSample(i, j)).path);
                trainingFeature(trainingCount, :) = vector.features;
                trainingLabel(trainingCount) = i;
            end
        end
        testingFeature = zeros(maxTestingSamplesPerCategory, dimensions);
        testingLabel = zeros(1, maxTestingSamplesPerCategory);
        for s = 1: 1: len
            options = ['-c ' num2str(C(s)) ', -q'];
            data(s).model = train(double(trainingLabel)', sparse(trainingFeature), options); %#ok<AGROW>
            data(s).testingResult = []; %#ok<AGROW>
        end
        for i = 1: 1: categories
            testingCount = 0;
            for j = 1: 1: testingSamplesPerCategory(i)
                testingCount = testingCount + 1;
                vector = load(feature.cate(i).image(testingSample(i, j)).path);
                testingFeature(testingCount, :) = vector.features;
                testingLabel(testingCount) = i;
            end
            for s = 1: 1: len
                fprintf('%d %d : ', i, s);
                testingResult_ = predict(testingLabel(1: testingCount)', sparse(testingFeature(1: testingCount, :)), data(s).model);
                data(s).testingResult = cat(1, data(s).testingResult, testingResult_); %#ok<AGROW>
            end
        end
        %testingResult = predict(testingLabel', sparse(testingFeature), model);
        fprintf('Classification based on SVM is finished. Statistics will be done.\n');
        for s = 1: 1: len
            testingCount = 0;
            confusionMatrix = zeros(categories, categories);
            correctSamples = 0;
            for i = 1: 1: categories
                for j = 1: 1: testingSamplesPerCategory(i)
                    testingCount = testingCount + 1;
                    confusionMatrix(i, data(s).testingResult(testingCount)) = ...
                        confusionMatrix(i, data(s).testingResult(testingCount)) + 1;
                end
                accuracyPerCategory(s, t, i) = confusionMatrix(i, i) / testingSamplesPerCategory(i);
                correctSamples = correctSamples + confusionMatrix(i, i);
                fprintf('    Category #%03d: The accuracy is %f%%. (Name is %s)\n', ...
                    i, accuracyPerCategory(s, t, i) * 100, feature.cate(i).name);
            end
            cateAcc(s, t) = mean(accuracyPerCategory(s, t, :));
            imageAcc(s, t) = correctSamples / testingSamples;
            fprintf('The mean accuracy by categories is %f%%.\n', cateAcc(s, t) * 100);
            fprintf('The mean accuracy by images is %f%%.\n', imageAcc(s, t) * 100);
            fprintf('Testing process No.%d is finished.\n', t);
        end
    end
    for s = 1: 1: len
        fprintf('All the testing is finished.\n');
        fprintf('The mean accuracy of all rounds by categories is %f%%.\n', ...
            mean(cateAcc(s, :)) * 100);
        fprintf('The std deviation of all rounds by categories is %f%%.\n', ...
            std(cateAcc(s, :)) * 100);
        fprintf('The mean accuracy of all rounds by images is %f%%.\n', ...
            mean(imageAcc(s, :)) * 100);
        fprintf('The std deviation of all rounds by images is %f%%.\n', ...
            std(imageAcc(s, :)) * 100);
        fprintf('All the testing is finished.\n');
        accuracy(s) = mean(cateAcc(s, :));
    end
    for s = 1: 1: len
        resultFile = [resultFileName num2str(C(s)) resultFileExpansion];
        file = fullfile(resultPath, resultFile);
        output = fopen(file, 'w');
        for t = 1: 1: times
            fprintf(output, 'Testing process No.%d begins.\r\n', t);
            for i = 1: 1: categories
                fprintf(output, '    Category #%03d: The accuracy is %f%%. (Name is %s)\r\n', ...
                    i, accuracyPerCategory(s, t, i) * 100, feature.cate(i).name);
            end
            fprintf(output, 'The mean accuracy by categories is %f%%.\r\n', cateAcc(s, t) * 100);
            fprintf(output, 'The mean accuracy by images is %f%%.\r\n', imageAcc(s, t) * 100);
            fprintf(output, 'Testing process No.%d is finished.\r\n', t);
            fprintf(output, '\r\n');
        end
        fprintf(output, 'All the testing is finished.\r\n');
        fprintf(output, 'The mean accuracy of all rounds by categories is %f%%.\r\n', ...
            mean(cateAcc(s, :)) * 100);
        fprintf(output, 'The std deviation of all rounds by categories is %f%%.\r\n', ...
            std(cateAcc(s, :)) * 100);
        fprintf(output, 'The mean accuracy of all rounds by images is %f%%.\r\n', ...
            mean(imageAcc(s, :)) * 100);
        fprintf(output, 'The std deviation of all rounds by images is %f%%.\r\n', ...
            std(imageAcc(s, :)) * 100);
        fprintf(output, 'All the testing is finished.\r\n');
        fclose(output);
    end
else
    accuracy = [];
    fprintf('Results are existed.\n');
end

finished = true;

end
