function [descriptor, finished] = LoadDescriptors(descriptorPath, maxImagesPerCategory, loadSIFT)

descriptor.categories = 0;
descriptor.images = 0;

if (exist(descriptorPath, 'dir'))
    fprintf('Begin loading descriptors of all images.\n');
    categoryDirectories = dir(descriptorPath);
    for i = 1: 1: length(categoryDirectories)
        categoryName = categoryDirectories(i).name;
        if (~strcmp(categoryName, '.') && ~strcmp(categoryName, '..'))
            descriptor.categories = descriptor.categories + 1;
            ca = descriptor.categories;
            descriptor.cate(ca).images = 0;
            descriptor.cate(ca).name = categoryName;
            imageDirectory = fullfile(descriptorPath, categoryName);
            imageFiles = dir(fullfile(imageDirectory, '*.mat'));
            count = length(imageFiles);
            perm = randperm(count);
            if (maxImagesPerCategory >= 0 && maxImagesPerCategory < count)
                count = maxImagesPerCategory;
            end
            for j = 1: 1: count
                descriptor.cate(ca).images = descriptor.cate(ca).images + 1;
                im = descriptor.cate(ca).images;
                [~, name_] = fileparts(imageFiles(perm(j)).name);
                descriptor.cate(ca).image(im).name = name_;
                descriptor.cate(ca).image(im).path = fullfile(descriptorPath, categoryName, imageFiles(perm(j)).name);
                if (loadSIFT)
                    SIFTDescriptor = load(descriptor.cate(ca).image(im).path);
                    descriptor.cate(ca).image(im).SIFT = SIFTDescriptor.SIFT;
                end
            end
            descriptor.images = descriptor.images + descriptor.cate(ca).images;
            if (descriptor.images > 0)
                fprintf('Descriptors of category %d are successfully loaded. %d images in total.\n', ...
                    descriptor.categories, descriptor.images);
            end
        end
    end
    if (descriptor.images > 0)
        fprintf('Descriptors are successfully loaded. %d categories and %d images in total.\n', ...
            descriptor.categories, descriptor.images);
    else
        fprintf('Testing process is finished.\n');
    end
    finished = true;
else
    fprintf('Bad descriptor directory. Please check if %s exists.\n', descriptorPath);
    finished = false;
end

end
